"""
Music package

@author: Ronny Andersson (ronny@andersson.tk)
@copyright: (c) 2016 Ronny Andersson
@license: MIT
"""

# Local folders
from . import scales, spn

__all__ = [
    'scales',
    'spn',
    ]
